Attribute VB_Name = "modWmiCore"
' **************************************************
' modWmiCore Functions
' L.J. Johnson, Slightly Tilted Software
' 03-11-2002
' Copyright  2002 Slightly Tilted Software
' **************************************************

Option Explicit

' **************************************************
' This function will return the top-level object for
'    either the local or remote machine. For the local
'    machine, pass in a blank string ("").
' **************************************************
Function GetTopObject(ByVal strServername As String) As SWbemServices
On Error Resume Next
   Dim objWbem                         As SWbemServices
   Dim strDesc                         As String
   
   Err.Clear

   If strServername = "" Then
      Set objWbem = GetObject("winmgmts:{impersonationLevel=impersonate}")
      'Set objWbem = GetObject("winmgmts:")
      If Err.Number <> 0 Then
         strDesc = "Error retrieving top-level WMI object for the local server" & _
                   "(Error: " & Err.Number & "), " & vbCrLf & _
                   "Description: " & Err.Description
         Err.Raise Number:=Err.Number, _
                   Source:="modWmiCore.GetTopObject", _
                   Description:=strDesc
         Exit Function
      Else
          Set GetTopObject = objWbem
      End If

   Else
      strServername = Trim$(strServername)
      If Left$(strServername, 2) <> "\\" Then
         strServername = "\\" & strServername
      End If

      Set objWbem = GetObject("winmgmts:{impersonationLevel=impersonate}" & strServername)
      If Err.Number <> 0 Then
         strDesc = "Error retrieving top-level WMI object for the computer, " & _
                   strServername & vbCrLf & _
                   "(Error: " & Err.Number & "), " & vbCrLf & _
                   "Description: " & Err.Description
         Err.Raise Number:=Err.Number, _
                   Source:="modWmiCore.GetTopObject", _
                   Description:=strDesc
         Exit Function
      Else
          Set GetTopObject = objWbem
      End If

   End If

End Function

' **************************************************
' This function will return the top-level object for
'    either the remote machine. The username and
'    password must be correct for a user that has the
'    appropriate rights on that remote machine
' **************************************************
Function GetTopObjectWithLogon(ByVal strServername As String, _
                               ByVal strUserID As String, _
                               ByVal strPassword As String) As SWbemServices
On Error Resume Next
   Dim objWbemLocator                  As SWbemLocator
   Dim objServices                     As SWbemServices
   Dim strDesc                         As String
   Const aCall                         As Long = 3
   Const iImpersonate                  As Long = 3

   Err.Clear

   ' Create the Locator object
   Set objWbemLocator = CreateObject("WbemScripting.SWbemLocator")

   ' Connect to the root-level object on the remote computer
   ' with the UserID and password that are passed.
   ' If no UserID, then assume current creditials
   If strUserID = "" Then
      Set objServices = objWbemLocator.ConnectServer(strServername, "Root\CIMV2")
   Else
      Set objServices = objWbemLocator.ConnectServer(strServername, "Root\CIMV2", strUserID, strPassword)
   End If

   If Err.Number <> 0 Then
      strDesc = "Error connecting to the computer, " & _
                strServername & vbCrLf & _
                "(Error: " & Err.Number & "), " & vbCrLf & _
                "Description: " & Err.Description
      Err.Raise Number:=Err.Number, _
                Source:="modWmiCore.GetTopObjectWithLogon", _
                Description:=strDesc
      Exit Function
   End If
   
   objServices.Security_.AuthenticationLevel = aCall
   objServices.Security_.ImpersonationLevel = iImpersonate
 
   If Err.Number <> 0 Then
      strDesc = "Error setting Authentication and " & vbCrLf & _
                "Impersonation Levels for the computer, " & _
                strServername & vbCrLf & _
                "(Error: " & Err.Number & "), " & vbCrLf & _
                "Description: " & Err.Description
      Err.Raise Number:=Err.Number, _
                Source:="modWmiCore.GetTopObjectWithLogon", _
                Description:=strDesc
      Exit Function
   End If

   Set GetTopObjectWithLogon = objServices

End Function

' **************************************************
' Given the root object of either a local or remote
'    machine, enumerate the properties of the
'    instance name you are passed
' **************************************************
Public Sub EnumInstancesOf(ByRef objRoot As SWbemServices, _
                           ByVal strInstanceName As String)
On Error Resume Next
   Dim objObjectSet                    As SWbemObjectSet
   Dim objObject                       As SWbemObject
   Dim objProperty                     As SWbemProperty

   Set objObjectSet = objRoot.InstancesOf(strInstanceName)
   
   For Each objObject In objObjectSet
      For Each objProperty In objObject.Properties_
         With objProperty
       ' Some items may produce errors
       ' Thus, the Resume Next at the
       ' top of the function
       Debug.Print .Name & " -- " & .Value
    End With
      Next
   Next

End Sub
